@@ -3,18 +3,20 @@ around the map."""
 
 from __future__ import annotations
 
-import json
 import os
 import shutil
 from copy import deepcopy
+from typing import Any
 
 import cv2
 import numpy as np
+from tqdm import tqdm
+from trimesh import Trimesh
 
 from maps4fs.generator.component.base.component_image import ImageComponent
 from maps4fs.generator.component.base.component_mesh import MeshComponent
+from maps4fs.generator.component.dem import DEM
 from maps4fs.generator.component.texture import Texture
-from maps4fs.generator.dem import DEM
 from maps4fs.generator.settings import Parameters
 
 
@@ -44,6 +46,7 @@ class Background(MeshComponent, ImageComponent):
 
         self.background_size = self.map_size + Parameters.BACKGROUND_DISTANCE * 2
         self.rotated_size = int(self.background_size * output_size_multiplier)
+        self.mesh_info: list[dict[str, Any]] = []
 
         self.background_directory = os.path.join(self.map_directory, "background")
         self.water_directory = os.path.join(self.map_directory, "water")
@@ -101,6 +104,56 @@ class Background(MeshComponent, ImageComponent):
         if self.map.background_settings.generate_water:
             self.generate_water_resources_obj()
 
+    def create_foundations(self, dem_image: np.ndarray) -> np.ndarray:
+        """Creates foundations for buildings based on the DEM data.
+
+        Arguments:
+            dem_image (np.ndarray): The DEM data as a numpy array.
+
+        Returns:
+            np.ndarray: The DEM data with the foundations added.
+        """
+        buildings = self.get_infolayer_data(Parameters.TEXTURES, Parameters.BUILDINGS)
+        if not buildings:
+            self.logger.warning("Buildings data not found in textures info layer.")
+            return dem_image
+
+        self.logger.debug("Found %s buildings in textures info layer.", len(buildings))
+
+        for building in tqdm(buildings, desc="Creating foundations", unit="building"):
+            try:
+                fitted_building = self.fit_object_into_bounds(
+                    polygon_points=building, angle=self.rotation
+                )
+            except ValueError as e:
+                self.logger.debug(
+                    "Building could not be fitted into the map bounds with error: %s",
+                    e,
+                )
+                continue
+
+            # 1. Read the pixel values from the DEM image.
+            # 2. Calculate the average pixel value of the building area.
+            # 3. Set the pixel values in the DEM to the average pixel value.
+
+            building_np = self.polygon_points_to_np(fitted_building)
+            mask = np.zeros(dem_image.shape, dtype=np.uint8)
+
+            try:
+                cv2.fillPoly(mask, [building_np], 255)  # type: ignore
+            except Exception as e:
+                self.logger.debug("Could not create mask for building with error: %s", e)
+                continue
+
+            mean_value = cv2.mean(dem_image, mask=mask)[0]
+            mean_value = np.round(mean_value).astype(dem_image.dtype)
+            self.logger.debug("Mean value of the building area: %s", mean_value)
+
+            # Set the pixel values in the DEM to the average pixel value.
+            dem_image[mask == 255] = mean_value
+
+        return dem_image
+
     def make_copy(self, dem_path: str, dem_name: str) -> None:
         """Copies DEM data to additional DEM file.
 
@@ -144,6 +197,7 @@ class Background(MeshComponent, ImageComponent):
 
         dem_info_sequence = self.dem.info_sequence()
         data["DEM"] = dem_info_sequence
+        data["Mesh"] = self.mesh_info
         return data  # type: ignore
 
     def qgis_sequence(self) -> None:
@@ -198,6 +252,9 @@ class Background(MeshComponent, ImageComponent):
             self.logger.debug("Not resized DEM saved: %s", save_path)
             return save_path
 
+        if self.map.dem_settings.add_foundations:
+            dem_data = self.create_foundations(dem_data)
+
         output_size = self.map_size + 1
 
         main_dem_path = self.game.dem_file_path(self.map_directory)
@@ -238,7 +295,7 @@ class Background(MeshComponent, ImageComponent):
         mesh = self.mesh_from_np(
             dem_data,
             include_zeros=include_zeros,
-            z_scaling_factor=self.get_z_scaling_factor(),
+            z_scaling_factor=self.get_z_scaling_factor(ignore_height_scale_multiplier=True),
             resize_factor=self.map.background_settings.resize_factor,
             apply_decimation=self.map.background_settings.apply_decimation,
             decimation_percent=self.map.background_settings.decimation_percent,
@@ -247,6 +304,11 @@ class Background(MeshComponent, ImageComponent):
             remove_size=self.map_size,
         )
 
+        try:
+            self.update_mesh_info(save_path, mesh)
+        except Exception as e:
+            self.logger.error("Could not update mesh info: %s", e)
+
         mesh.export(save_path)
         self.logger.debug("Obj file saved: %s", save_path)
 
@@ -254,6 +316,29 @@ class Background(MeshComponent, ImageComponent):
             mesh.apply_scale([0.5, 0.5, 0.5])
             self.mesh_to_stl(mesh, save_path=self.stl_preview_path)
 
+    def update_mesh_info(self, save_path: str, mesh: Trimesh) -> None:
+        """Updates the mesh info with the data from the mesh.
+
+        Arguments:
+            save_path (str): The path where the mesh is saved.
+            mesh (Trimesh): The mesh to get the data from.
+        """
+        filename = os.path.splitext(os.path.basename(save_path))[0]
+        x_size, y_size, z_size = mesh.extents
+        x_center, y_center, z_center = mesh.centroid
+
+        entry = {
+            "name": filename,
+            "x_size": round(x_size, 4),
+            "y_size": round(y_size, 4),
+            "z_size": round(z_size, 4),
+            "x_center": round(x_center, 4),
+            "y_center": round(y_center, 4),
+            "z_center": round(z_center, 4),
+        }
+
+        self.mesh_info.append(entry)
+
     def previews(self) -> list[str]:
         """Returns the path to the image previews paths and the path to the STL preview file.
 
@@ -347,13 +432,11 @@ class Background(MeshComponent, ImageComponent):
 
     def create_background_textures(self) -> None:
         """Creates background textures for the map."""
-        if not os.path.isfile(self.game.texture_schema):
-            self.logger.warning("Texture schema file not found: %s", self.game.texture_schema)
+        layers_schema = self.map.texture_schema
+        if not layers_schema:
+            self.logger.warning("No texture schema found.")
             return
 
-        with open(self.game.texture_schema, "r", encoding="utf-8") as f:
-            layers_schema = json.load(f)
-
         background_layers = []
         for layer in layers_schema:
             if layer.get("background") is True:
@@ -402,6 +485,9 @@ class Background(MeshComponent, ImageComponent):
         if not self.water_resources_path:
             self.logger.warning("Water resources texture not found.")
             return
+        if not os.path.isfile(self.water_resources_path):
+            self.logger.warning("Water resources texture was not generated, skipping subtraction.")
+            return
 
         water_resources_image = cv2.imread(self.water_resources_path, cv2.IMREAD_UNCHANGED)
         dem_image = cv2.imread(self.output_path, cv2.IMREAD_UNCHANGED)
@@ -424,6 +510,18 @@ class Background(MeshComponent, ImageComponent):
         cv2.imwrite(self.output_path, dem_image)
         self.logger.debug("Water depth subtracted from DEM data: %s", self.output_path)
 
+    def _get_blur_power(self) -> int:
+        """Returns the blur power for the water resources to apply Gaussian blur.
+
+        Returns:
+            int: The blur power for the water resources.
+        """
+        blur_power = max(3, min(self.map.background_settings.water_blurriness, 99))
+        if blur_power % 2 == 0:
+            blur_power += 1
+
+        return blur_power
+
     def generate_water_resources_obj(self) -> None:
         """Generates 3D obj files based on water resources data."""
         if not os.path.isfile(self.water_resources_path):
@@ -447,6 +545,13 @@ class Background(MeshComponent, ImageComponent):
         # Single channeled 16 bit DEM image of terrain.
         background_dem = cv2.imread(self.not_substracted_path, cv2.IMREAD_UNCHANGED)
 
+        if self.map.background_settings.water_blurriness:
+            # Apply Gaussian blur to the background dem.
+            blur_power = self._get_blur_power()
+            background_dem = cv2.GaussianBlur(
+                background_dem, (blur_power, blur_power), sigmaX=blur_power, sigmaY=blur_power
+            )
+
         # Remove all the values from the background dem where the plane_water is 0.
         background_dem[plane_water == 0] = 0
 
