@@ -7,130 +7,25 @@ import os
 import shutil
 from typing import Any, Generator
 
-from pydantic import BaseModel
+from geopy.geocoders import Nominatim
 
-from maps4fs.generator.component import Component
+from maps4fs.generator.component import Background, Component, Layer, Texture
+from maps4fs.generator.dtm.dtm import DTMProvider, DTMProviderSettings
 from maps4fs.generator.game import Game
+from maps4fs.generator.settings import (
+    BackgroundSettings,
+    DEMSettings,
+    GRLESettings,
+    I3DSettings,
+    SatelliteSettings,
+    SharedSettings,
+    SplineSettings,
+    TextureSettings,
+)
+from maps4fs.generator.statistics import send_advanced_settings, send_main_settings
 from maps4fs.logger import Logger
 
 
-class SettingsModel(BaseModel):
-    """Base class for settings models. It provides methods to convert settings to and from JSON."""
-
-    @classmethod
-    def all_settings_to_json(cls) -> dict[str, dict[str, Any]]:
-        """Get all settings of the current class and its subclasses as a dictionary.
-
-        Returns:
-            dict[str, dict[str, Any]]: Dictionary with settings of the current class and its
-                subclasses.
-        """
-        all_settings = {}
-        for subclass in cls.__subclasses__():
-            all_settings[subclass.__name__] = subclass().model_dump()
-
-        return all_settings
-
-    @classmethod
-    def all_settings_from_json(cls, data: dict) -> dict[str, SettingsModel]:
-        """Create settings instances from JSON data.
-
-        Arguments:
-            data (dict): JSON data.
-
-        Returns:
-            dict[str, Type[SettingsModel]]: Dictionary with settings instances.
-        """
-        settings = {}
-        for subclass in cls.__subclasses__():
-            settings[subclass.__name__] = subclass(**data[subclass.__name__])
-
-        return settings
-
-
-class DEMSettings(SettingsModel):
-    """Represents the advanced settings for DEM component.
-
-    Attributes:
-        auto_process (bool): use the auto preset to change the multiplier.
-        multiplier (int): multiplier for the heightmap, every pixel will be multiplied by this
-            value.
-        blur_radius (int): radius of the blur filter.
-        plateau (int): plateau height, will be added to each pixel.
-        water_depth (int): water depth, will be subtracted from each pixel where the water
-            is present.
-    """
-
-    auto_process: bool = True
-    multiplier: int = 1
-    blur_radius: int = 35
-    plateau: int = 0
-    water_depth: int = 0
-
-
-class BackgroundSettings(SettingsModel):
-    """Represents the advanced settings for background component.
-
-    Attributes:
-        generate_background (bool): generate obj files for the background terrain.
-        generate_water (bool): generate obj files for the water.
-        resize_factor (int): resize factor for the background terrain and water.
-            It will be used as 1 / resize_factor of the original size.
-    """
-
-    generate_background: bool = True
-    generate_water: bool = True
-    resize_factor: int = 8
-
-
-class GRLESettings(SettingsModel):
-    """Represents the advanced settings for GRLE component.
-
-    Attributes:
-        farmland_margin (int): margin around the farmland.
-        random_plants (bool): generate random plants on the map or use the default one.
-    """
-
-    farmland_margin: int = 0
-    random_plants: bool = True
-
-
-class I3DSettings(SettingsModel):
-    """Represents the advanced settings for I3D component.
-
-    Attributes:
-        forest_density (int): density of the forest (distance between trees).
-    """
-
-    forest_density: int = 10
-
-
-class TextureSettings(SettingsModel):
-    """Represents the advanced settings for texture component.
-
-    Attributes:
-        dissolve (bool): dissolve the texture into several images.
-        fields_padding (int): padding around the fields.
-        skip_drains (bool): skip drains generation.
-    """
-
-    dissolve: bool = True
-    fields_padding: int = 0
-    skip_drains: bool = False
-
-
-class SplineSettings(SettingsModel):
-    """Represents the advanced settings for spline component.
-
-    Attributes:
-        spline_density (int): the number of extra points that will be added between each two
-            existing points.
-    """
-
-    spline_density: int = 2
-
-
-# pylint: disable=R0913, R0902, R0914
 class Map:
     """Class used to generate map using all components.
 
@@ -142,9 +37,11 @@ class Map:
         logger (Any): Logger instance
     """
 
-    def __init__(  # pylint: disable=R0917, R0915
+    def __init__(
         self,
         game: Game,
+        dtm_provider: DTMProvider,
+        dtm_provider_settings: DTMProviderSettings | None,
         coordinates: tuple[float, float],
         size: int,
         rotation: int,
@@ -157,6 +54,7 @@ class Map:
         i3d_settings: I3DSettings = I3DSettings(),
         texture_settings: TextureSettings = TextureSettings(),
         spline_settings: SplineSettings = SplineSettings(),
+        satellite_settings: SatelliteSettings = SatelliteSettings(),
         **kwargs,
     ):
         if not logger:
@@ -173,14 +71,35 @@ class Map:
         self.rotated_size = int(size * rotation_multiplier)
 
         self.game = game
+        self.dtm_provider = dtm_provider
+        self.dtm_provider_settings = dtm_provider_settings
         self.components: list[Component] = []
         self.coordinates = coordinates
         self.map_directory = map_directory
 
-        self.logger.info("Game was set to %s", game.code)
+        try:
+            main_settings = {
+                "game": game.code,
+                "latitude": coordinates[0],
+                "longitude": coordinates[1],
+                "country": self.get_country_by_coordinates(),
+                "size": size,
+                "rotation": rotation,
+                "dtm_provider": dtm_provider.name(),
+                "custom_osm": bool(custom_osm),
+                "is_public": kwargs.get("is_public", False),
+            }
+            send_main_settings(main_settings)
+        except Exception as e:
+            self.logger.error("Error sending main settings: %s", e)
+
+        log_entry = ""
+        log_entry += f"Map instance created for Game: {game.code}. "
+        log_entry += f"Coordinates: {coordinates}. Size: {size}. Rotation: {rotation}. "
+        log_entry += f"DTM provider is {dtm_provider.name()}. "
 
         self.custom_osm = custom_osm
-        self.logger.info("Custom OSM file: %s", custom_osm)
+        log_entry += f"Custom OSM file: {custom_osm}. "
 
         # Make a copy of a custom osm file to the map directory, so it will be
         # included in the output archive.
@@ -190,21 +109,56 @@ class Map:
             self.logger.debug("Custom OSM file copied to %s", copy_path)
 
         self.dem_settings = dem_settings
-        self.logger.info("DEM settings: %s", dem_settings)
-        self.background_settings = background_settings
-        self.logger.info("Background settings: %s", background_settings)
-        self.grle_settings = grle_settings
-        self.logger.info("GRLE settings: %s", grle_settings)
-        self.i3d_settings = i3d_settings
-        self.logger.info("I3D settings: %s", i3d_settings)
-        self.texture_settings = texture_settings
-        self.logger.info("Texture settings: %s", texture_settings)
-        self.spline_settings = spline_settings
-        self.logger.info("Spline settings: %s", spline_settings)
+        log_entry += f"DEM settings: {dem_settings}. "
+        if self.dem_settings.water_depth > 0:
+            # Make sure that the plateau value is >= water_depth
+            self.dem_settings.plateau = max(
+                self.dem_settings.plateau, self.dem_settings.water_depth
+            )
 
+        self.background_settings = background_settings
+        log_entry += f"Background settings: {background_settings}. "
+        self.grle_settings = grle_settings
+        log_entry += f"GRLE settings: {grle_settings}. "
+        self.i3d_settings = i3d_settings
+        log_entry += f"I3D settings: {i3d_settings}. "
+        self.texture_settings = texture_settings
+        log_entry += f"Texture settings: {texture_settings}. "
+        self.spline_settings = spline_settings
+        log_entry += f"Spline settings: {spline_settings}. "
+        self.satellite_settings = satellite_settings
+
+        self.logger.info(log_entry)
         os.makedirs(self.map_directory, exist_ok=True)
         self.logger.debug("Map directory created: %s", self.map_directory)
 
+        settings = [
+            dem_settings,
+            background_settings,
+            grle_settings,
+            i3d_settings,
+            texture_settings,
+            spline_settings,
+            satellite_settings,
+        ]
+
+        settings_json = {}
+
+        for setting in settings:
+            settings_json[setting.__class__.__name__] = setting.model_dump()
+
+        try:
+            send_advanced_settings(settings_json)
+        except Exception as e:
+            self.logger.error("Error sending advanced settings: %s", e)
+
+        save_path = os.path.join(self.map_directory, "generation_settings.json")
+
+        with open(save_path, "w", encoding="utf-8") as file:
+            json.dump(settings_json, file, indent=4)
+
+        self.shared_settings = SharedSettings()
+
         self.texture_custom_schema = kwargs.get("texture_custom_schema", None)
         if self.texture_custom_schema:
             save_path = os.path.join(self.map_directory, "texture_custom_schema.json")
@@ -219,19 +173,36 @@ class Map:
                 json.dump(self.tree_custom_schema, file, indent=4)
             self.logger.debug("Tree custom schema saved to %s", save_path)
 
+        self.custom_background_path = kwargs.get("custom_background_path", None)
+        if self.custom_background_path:
+            save_path = os.path.join(self.map_directory, "custom_background.png")
+            shutil.copyfile(self.custom_background_path, save_path)
+
         try:
             shutil.unpack_archive(game.template_path, self.map_directory)
             self.logger.debug("Map template unpacked to %s", self.map_directory)
         except Exception as e:
             raise RuntimeError(f"Can not unpack map template due to error: {e}") from e
 
+    @property
+    def texture_schema(self) -> list[dict[str, Any]] | None:
+        """Return texture schema (custom if provided, default otherwise).
+
+        Returns:
+            list[dict[str, Any]] | None: Texture schema.
+        """
+        if self.texture_custom_schema:
+            return self.texture_custom_schema
+        with open(self.game.texture_schema, "r", encoding="utf-8") as file:
+            return json.load(file)
+
     def generate(self) -> Generator[str, None, None]:
         """Launch map generation using all components. Yield component names during the process.
 
         Yields:
             Generator[str, None, None]: Component names.
         """
-        self.logger.info(
+        self.logger.debug(
             "Starting map generation. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
             self.game.code,
             self.coordinates,
@@ -258,7 +229,7 @@ class Map:
 
             try:
                 component.process()
-            except Exception as e:  # pylint: disable=W0718
+            except Exception as e:
                 self.logger.error(
                     "Error processing component %s: %s",
                     component.__class__.__name__,
@@ -268,7 +239,7 @@ class Map:
 
             try:
                 component.commit_generation_info()
-            except Exception as e:  # pylint: disable=W0718
+            except Exception as e:
                 self.logger.error(
                     "Error committing generation info for component %s: %s",
                     component.__class__.__name__,
@@ -276,7 +247,7 @@ class Map:
                 )
                 raise e
 
-        self.logger.info(
+        self.logger.debug(
             "Map generation completed. Game code: %s. Coordinates: %s, size: %s. Rotation: %s.",
             self.game.code,
             self.coordinates,
@@ -298,6 +269,63 @@ class Map:
                 return component
         return None
 
+    def get_texture_component(self) -> Texture | None:
+        """Get texture component.
+
+        Returns:
+            Texture | None: Texture instance or None if not found.
+        """
+        component = self.get_component("Texture")
+        if not isinstance(component, Texture):
+            return None
+        return component
+
+    def get_background_component(self) -> Background | None:
+        """Get background component.
+
+        Returns:
+            Background | None: Background instance or None if not found.
+        """
+        component = self.get_component("Background")
+        if not isinstance(component, Background):
+            return None
+        return component
+
+    def get_texture_layer(self, by_usage: str | None = None) -> Layer | None:
+        """Get texture layer by usage.
+
+        Arguments:
+            by_usage (str, optional): Texture usage.
+
+        Returns:
+            Layer | None: Texture layer instance or None if not found.
+        """
+        texture_component = self.get_texture_component()
+        if not texture_component:
+            return None
+        if by_usage:
+            return texture_component.get_layer_by_usage(by_usage)
+        return None
+
+    def get_texture_layers(
+        self,
+        by_usage: str | None = None,
+    ) -> None | list[Layer]:
+        """Get texture layers by usage.
+
+        Arguments:
+            by_usage (str, optional): Texture usage.
+
+        Returns:
+            None | list[Layer]: List of texture layers.
+        """
+        texture_component = self.get_texture_component()
+        if not texture_component:
+            return None
+        if by_usage:
+            return texture_component.get_layers_by_usage(by_usage)
+        return None
+
     def previews(self) -> list[str]:
         """Get list of preview images.
 
@@ -308,7 +336,7 @@ class Map:
         for component in self.components:
             try:
                 previews.extend(component.previews())
-            except Exception as e:  # pylint: disable=W0718
+            except Exception as e:
                 self.logger.error(
                     "Error getting previews for component %s: %s",
                     component.__class__.__name__,
@@ -327,11 +355,27 @@ class Map:
             str: Path to the archive.
         """
         archive_path = shutil.make_archive(archive_path, "zip", self.map_directory)
-        self.logger.info("Map packed to %s.zip", archive_path)
+        self.logger.debug("Map packed to %s.zip", archive_path)
         if remove_source:
             try:
                 shutil.rmtree(self.map_directory)
                 self.logger.debug("Map directory removed: %s", self.map_directory)
-            except Exception as e:  # pylint: disable=W0718
+            except Exception as e:
                 self.logger.debug("Error removing map directory %s: %s", self.map_directory, e)
         return archive_path
+
+    def get_country_by_coordinates(self) -> str:
+        """Get country name by coordinates.
+
+        Returns:
+            str: Country name.
+        """
+        try:
+            geolocator = Nominatim(user_agent="maps4fs")
+            location = geolocator.reverse(self.coordinates, language="en")
+            if location and "country" in location.raw["address"]:
+                return location.raw["address"]["country"]
+        except Exception as e:
+            self.logger.error("Error getting country name by coordinates: %s", e)
+            return "Unknown"
+        return "Unknown"
